import Select, { StylesConfig, components } from 'react-select';
import styled from 'styled-components';
import { motion } from 'framer-motion';
import theme from '../../../../theme';
import Icon from '../../atoms/Icon';

const styles: StylesConfig = {
  control: (styles, { isFocused, isDisabled }) => ({
    display: 'flex',
    backgroundColor: 'white',
    fontSize: 16,
    fontFamily: theme.fontFamily.bold,
    color: 'white',
  }),
  indicatorSeparator: styles => ({ ...styles, display: 'none' }),
  dropdownIndicator: styles => ({
    ...styles,
    color: theme.colors.gray500,
  }),
  container: (styles, { isFocused, isDisabled }) => ({
    ...styles,
    border: `1px solid ${isFocused ? '#4e73f8' : theme.colors.gray500}`,
    borderRadius: 8,
    padding: '8px 16px',
    background: isDisabled ? theme.colors.gray50 : theme.colors.white,
  }),
  valueContainer: (styles, { isDisabled }) => ({
    ...styles,
    padding: 0,
    margin: 0,
    background: isDisabled ? theme.colors.gray50 : theme.colors.white,
  }),
  singleValue: styles => ({ ...styles, color: theme.colors.gray500 }),
  input: (styles, { isDisabled }) => ({
    ...styles,
    padding: 0,
    margin: 0,
    background: isDisabled ? theme.colors.gray50 : theme.colors.white,
  }),
  option: styles => null, // 하단에서 커스텀 SelectOption을 재정의함
  menu: styles => ({
    ...styles,
    left: 0,
    border: `1px solid ${theme.colors.gray500}`,
  }),
  menuList: styles => ({
    ...styles,
    padding: '24px 16px',
  }),
};

interface OptionType {
  value: string | number;
  label: string | React.ReactNode;
  isDisabled?: boolean;
}

interface SelectInputProps {
  isDisabled?: boolean;
  options?: OptionType[];
  defaultMenuIsOpen?: boolean;
  placeholder?: string;
}

export default function SelectInput(props: SelectInputProps) {
  const { isDisabled, defaultMenuIsOpen, placeholder, options } = props;

  return (
    <Select
      closeMenuOnSelect={true}
      options={options}
      styles={styles}
      components={{
        DropdownIndicator,
        Option: SelectOption,
      }}
      isDisabled={isDisabled}
      isSearchable={false}
      placeholder={placeholder}
    />
  );
}

SelectInput.defaultProps = {
  isDisabled: false,
  options: [
    { value: 'strawberry', label: '딸기', isDisabled: false },
    { value: 'carrot', label: '당근', isDisabled: false },
    { value: 'watermelon', label: '수박', isDisabled: true },
    { value: 'melon', label: '메론', isDisabled: false },
  ],
  placeholder: '값을 선택해주세요',
};

const SelectOption = props => {
  const { label, value, isFocused, isDisabled, isSelected } = props;

  return (
    <SelectOptionStyled value={value} isFocused={isFocused} isDisabled={isDisabled} isSelected={isSelected} {...props}>
      {label}
      {isSelected && <Icon name="check" size={20} backgroundColor="transparent" color={'#4e73f8'} />}
    </SelectOptionStyled>
  );
};

const DropdownIndicator = props => {
  const isOpen = props.selectProps.menuIsOpen;
  const isDisabled = props.selectProps.isDisabled;

  return <Icon name={isOpen ? 'chevron-up' : 'chevron-down'} size={20} color={theme.colors.gray500} backgroundColor={isDisabled ? theme.colors.gray50 : theme.colors.white} />;
};

const SelectOptionBox = styled(motion.div)``;

const SelectOptionStyled = styled(components.Option)`
  display: flex;
  justify-content: space-between;
  cursor: pointer;
  padding: 9px 16px;
  border-radius: 8px;
  font-size: 16px;
  line-height: 1.2;
  font-family: ${({ isFocused, isSelected }) => (isFocused || isSelected ? theme.fontFamily.bold : theme.fontFamily.normal)};
  color: ${({ isFocused, isSelected, isDisabled }) => (isFocused || isSelected ? '#4e73f8' : isDisabled ? theme.colors.gray200 : theme.colors.gray300)};
  background-color: ${({ isFocused, isSelected, isDisabled }) => (isFocused || isSelected ? '#f2f4ff' : isDisabled ? theme.colors.gray50 : theme.colors.white)};
  & + & {
    margin-top: 8px;
  }
`;
