import styled from 'styled-components';
import { motion } from 'framer-motion';
import { produce } from 'immer';
import theme from '../../../../theme';
import { Title, Paragraph } from '../../atoms';
import { ParagraphProps } from '../../atoms/Paragraph/Paragraph';
import { TitleProps } from '../../atoms/Title/Title';
import { Badge } from '../../molecules';
import { BadgeProps } from '../../molecules/Badge/Badge';
import Button, { ButtonProps } from '../../molecules/Button/Button';
import { Stack } from '../../../layout';
import { isEmpty } from 'lodash-es';

export interface Feature3Props {
  imageUrls?: string[];
  thumbnail?: string;
  badgeProps?: BadgeProps;
  subTitleProps?: TitleProps;
  titleProps?: TitleProps;
  descriptionProps?: ParagraphProps;
  isMobile?: boolean;
  backgroundColor?: string;
  buttonProps?: ButtonProps;
  isReverse?: boolean;
}

export default function Feature3(props: Feature3Props) {
  const { imageUrls, thumbnail, badgeProps, subTitleProps, titleProps, descriptionProps, isMobile, backgroundColor, buttonProps, isReverse } = props;

  return (
    <Box backgroundColor={backgroundColor} isMobile={isMobile}>
      <Stack direction={isMobile ? 'column' : isReverse ? 'row-reverse' : 'row'} gap={isMobile ? 40 : 72} alignItems="center">
        <Stack direction="column" gap={isMobile ? 12 : 20}>
          {!isEmpty(imageUrls) && (
            <Stack gap={8}>
              {imageUrls?.map((imageUrl, index) => {
                return <IconImage key={index} src={imageUrl} />;
              })}
            </Stack>
          )}
          <Badge {...badgeProps} />
          <Title {...subTitleProps} isMobile={isMobile} />
          <Title {...titleProps} isMobile={isMobile} />
          <Paragraph {...descriptionProps} isMobile={isMobile} />
        </Stack>
        <Thumbnail src={thumbnail} isMobile={isMobile} />
      </Stack>
    </Box>
  );
}

Feature3.defaultProps = {
  thumbnail: 'https://picsum.photos/id/57/800/800',
  imageUrls: ['https://picsum.photos/id/1/32', 'https://picsum.photos/id/2/32', 'https://picsum.photos/id/3/32'],
  badgeProps: Badge.defaultProps,
  subTitleProps: produce(Title.defaultProps, draft => {
    draft.fontWeight = 'bold';
    draft.size = 'sm';
    draft.color = theme.colors.black;
    draft.content = 'Subtitle 01 서브타이틀 01';
  }),
  titleProps: produce(Title.defaultProps, draft => {
    draft.fontWeight = 'bold';
    draft.size = 'lg';
    draft.color = theme.colors.black;
    draft.content = 'Title 01 타이틀 01\nTitle 02 타이틀 02';
  }),
  descriptionProps: produce(Paragraph.defaultProps, draft => {
    draft.fontWeight = 'normal';
    draft.size = 'md';
    draft.color = theme.colors.gray400;
    draft.content = '기능을 설명하는 글입니다.기능을 설명하는 글입니다.기능을 설명하는 글입니다.';
  }),
  backgroundColor: 'transparent',
  buttonProps: produce(Button.defaultProps, draft => {
    draft.backgroundColor = theme.colors.white;
    draft.icon = 'chevron-right';
    draft.iconPosition = 'right';
    draft.color = theme.colors.gray500;
    draft.content = '포트폴리오 보러가기';
    draft.border.color = theme.colors.gray500;
    draft.border.width = 1;
  }),
  isMobile: false,
  isReverse: false,
};

const Box = styled(motion.div)<any>`
  display: flex;
  flex-direction: ${props => (props.isMobile ? 'column' : 'row')};
  background-color: ${props => props.backgroundColor};
  width: 100%;
  margin: 0 auto;
`;
const Thumbnail = styled(motion.img)<any>`
  width: ${props => (props.isMobile ? '100%' : 'calc(50% - (72px/2))')};
  border-radius: 16px;
`;

const IconImage = styled(motion.img)<any>`
  height: ${props => (props.isMobile ? '32px' : '40px')};
`;
