import { ReactNode, useEffect, useRef, useState } from 'react';
import { Swiper, Navigation, Pagination, Scrollbar, A11y } from 'swiper';
import { Swiper as SwiperReact, SwiperSlide } from 'swiper/react';
import 'swiper/swiper-bundle.min.css';
import { motion } from 'framer-motion';
import { isEmpty } from 'lodash-es';
import styled from 'styled-components';
import { produce } from 'immer';
import theme from '../../../../theme';
import IconButton, { IconButtonProps } from '../../molecules/IconButton/IconButton';
import Card1 from '../../organisms/Card1/Card1';

export interface CarouselProps {
  slides: ReactNode[];
  spaceBetween?: number;
  slidesPerView?: number;
  pagination?: {
    isHide?: boolean;
  };
  prevNavigation?: {
    isHideOnFirstPage?: boolean;
    buttonProps?: IconButtonProps;
  };
  nextNavigation?: {
    isHideOnLastPage?: boolean;
    buttonProps?: IconButtonProps;
  };
  padding?: {
    left?: number;
    top?: number;
    right?: number;
    bottom?: number;
  };
}

function Carousel1(props: CarouselProps) {
  const slides = !isEmpty(props.slides) ? props.slides : [<Card1 />, <Card1 />, <Card1 />, <Card1 />, <Card1 />, <Card1 />];
  const [swiper, setSwiper] = useState<Swiper>();
  /** Framer 버그로 인하여 추가함 (강제 리렌더링) */
  const [reRender, setReRender] = useState(0);

  const isShowPrevButton = !props?.prevNavigation?.isHideOnFirstPage || !swiper?.isBeginning;

  const isShowNextButton = !props?.nextNavigation?.isHideOnLastPage || !swiper?.isEnd;

  return (
    <CarouselBox padding={props?.padding}>
      <SwiperReact
        modules={[Navigation, Pagination, Scrollbar, A11y]}
        spaceBetween={props.spaceBetween}
        slidesPerView={props.slidesPerView}
        navigation={{ prevEl: null, nextEl: null }}
        pagination={{ clickable: true, enabled: false }}
        scrollbar={{ draggable: true, enabled: false }}
        onSwiper={setSwiper}
        onSlideChange={swiper => setReRender(prev => prev + 1)}
      >
        {slides.map((slide, index) => (
          <SwiperSlide key={index}>{slide}</SwiperSlide>
        ))}
      </SwiperReact>
      {isShowPrevButton && (
        <PreviousButton id="swiper-custom-button-prev" onClick={() => swiper?.slidePrev()} icon="chevron-left" iconSize={48} color={theme.colors.white} backgroundColor={theme.colors.black} opacity={0.2} {...props?.prevNavigation?.buttonProps} />
      )}
      {isShowNextButton && <NextButton onClick={() => swiper?.slideNext()} icon="chevron-right" iconSize={48} color={theme.colors.white} backgroundColor={theme.colors.black} opacity={0.2} {...props?.nextNavigation?.buttonProps} />}
    </CarouselBox>
  );
}

Carousel1.defaultProps = {
  slides: Array(8).fill(<Card1 />),
  spaceBetween: 16,
  slidesPerView: 3.5,
  padding: {
    top: 0,
    bottom: 0,
    left: 0,
    right: 0,
  },
  pagination: false,
  prevNavigation: {
    isHideOnFirstPage: true,
    buttonProps: produce(IconButton.defaultProps, draft => {
      draft.icon = 'chevron-left';
      draft.iconSize = 48;
      draft.color = theme.colors.white;
      draft.backgroundColor = theme.colors.black;
      draft.opacity = 0.2;
    }),
  },
  nextNavigation: {
    isHideOnLastPage: true,
    buttonProps: produce(IconButton.defaultProps, draft => {
      draft.icon = 'chevron-right';
      draft.iconSize = 48;
      draft.color = theme.colors.white;
      draft.backgroundColor = theme.colors.black;
      draft.opacity = 0.2;
    }),
  },
};

export default Carousel1;

const CarouselBox = styled(motion.div)<any>`
  position: relative;
  padding-left: ${props => props?.padding?.left}px;
  padding-top: ${props => props?.padding?.top}px;
  padding-bottom: ${props => props?.padding?.bottom}px;
  padding-right: ${props => props?.padding?.right}px;
  .swiper-wrapper {
    display: flex;
  }
`;

const PreviousButton = styled(IconButton)<any>`
  position: absolute;
  top: 50%;
  left: 20px;
  transform: translateY(-50%);
  z-index: 999;
`;

const NextButton = styled(IconButton)<any>`
  position: absolute;
  top: 50%;
  right: 20px;
  transform: translateY(-50%);
  z-index: 999;
`;
