import styled from 'styled-components';
import { motion } from 'framer-motion';
import { isEmpty } from 'lodash-es';
import { produce } from 'immer';
import theme from '../../../../theme';
import { Container, Stack } from '../../../layout';
import { Title, Paragraph } from '../../atoms';
import { ParagraphProps } from '../../atoms/Paragraph/Paragraph';
import { TitleProps } from '../../atoms/Title/Title';
import { Badge, Button } from '../../molecules';
import { BadgeProps } from '../../molecules/Badge/Badge';
import { ButtonProps } from '../../molecules/Button/Button';
import Feature3, { Feature3Props } from '../../organisms/Feature3/Feature3';

export interface FeatureSection4Props {
  imageUrls: string[];
  badgeProps: BadgeProps;
  subTitleProps: TitleProps;
  titleProps: TitleProps;
  descriptionProps: ParagraphProps;
  featuresProps: Feature3Props[];
  buttonsProps: ButtonProps[];
  backgroundImage?: string;
  backgroundColor?: string;
  isMobile?: boolean;
}

export default function FeatureSection4(props: FeatureSection4Props) {
  const { imageUrls, badgeProps, subTitleProps, titleProps, descriptionProps, featuresProps, buttonsProps, backgroundImage, backgroundColor, isMobile } = props;
  const isVisibleTitleArea = subTitleProps.isVisible || titleProps.isVisible || descriptionProps.isVisible;

  return (
    <Container isMobile={isMobile} direction="column" isCenter={true} backgroundColor={backgroundColor} backgroundImage={backgroundImage}>
      <Stack direction="column" gap={64}>
        {isVisibleTitleArea && (
          <Stack direction="column" gap={24} alignItems="center">
            {!isEmpty(imageUrls) && (
              <Stack gap={8} justifyContent="center">
                {imageUrls?.map((imageUrl, index) => {
                  return <IconImage key={index} src={imageUrl} isMobile={isMobile} />;
                })}
              </Stack>
            )}
            <Badge {...badgeProps} />
            <Stack direction="column" gap={8} alignItems="center">
              <Title {...subTitleProps} isMobile={isMobile} />
              <Title {...titleProps} isMobile={isMobile} />
              <Paragraph {...descriptionProps} isMobile={isMobile} />
            </Stack>
          </Stack>
        )}
        <Stack direction="column" gap={isMobile ? 80 : 120}>
          {featuresProps?.map((feature, index) => {
            return <Feature3 key={index} {...feature} isMobile={isMobile} />;
          })}
        </Stack>
        {!isEmpty(buttonsProps) && (
          <Stack gap={12} direction={isMobile ? 'column' : 'row'} justifyContent="center">
            {buttonsProps?.map((button, index) => {
              return (
                <ButtonBox>
                  <Button key={index} {...button} size={isMobile ? 'md' : 'xl'} />
                </ButtonBox>
              );
            })}
          </Stack>
        )}
      </Stack>
    </Container>
  );
}

FeatureSection4.defaultProps = {
  imageUrls: ['https://picsum.photos/id/1/32', 'https://picsum.photos/id/2/32', 'https://picsum.photos/id/3/32'],
  badgeProps: Badge.defaultProps,
  subTitleProps: produce(Title.defaultProps, draft => {
    draft.content = 'Subtitle 01 서브타이틀 01';
    draft.size = 'sm';
    draft.textAlign = 'center';
  }),
  titleProps: produce(Title.defaultProps, draft => {
    draft.content = 'Title 01 타이틀 01\nTitle 01 타이틀 01';
    draft.size = 'lg';
    draft.textAlign = 'center';
  }),
  descriptionProps: produce(Paragraph.defaultProps, draft => {
    draft.content = 'Body 입력 Body 입력 무엇을 구하지 꽃이 무엇이 천고에 천자만홍이 황금시대다. 피는 대한 따뜻한 꽃 가슴이 봄바람이다. 속에서 밥을 끓는 속잎나고.';
    draft.size = 'lg';
    draft.textAlign = 'center';
    draft.color = theme.colors.gray500;
  }),
  featuresProps: [{ ...Feature3.defaultProps, isReverse: true }, Feature3.defaultProps, { ...Feature3.defaultProps, isReverse: true }],
  buttonsProps: Array(1).fill(
    produce(Button.defaultProps, draft => {
      draft.isFullWidth = true;
      draft.size = 'xl';
    }),
  ),
  backgroundImage: 'https://picsum.photos/id/130/1600',
  backgroundColor: 'transparent',
  isMobile: false,
};

const ButtonBox = styled(motion.div)`
  flex: 1;
  width: 100%;
`;

const IconImage = styled(motion.img)<any>`
  height: ${props => (props.isMobile ? '32px' : '40px')};
`;
