import axios, { AxiosInstance } from 'axios';

class Api {
  private readonly api: AxiosInstance;
  private accessToken?: string;
  private loginPath = '/tutors/login';
  private loginApiPath = '/api/v1/auth/login';
  private tokenRefreshApiPath = '/api/v1/auth/token/refresh';
  private API_ROOT = process.env.NODE_ENV === 'production' ? 'https://api.com' : 'http://localhost:8080';

  constructor() {
    this.api = axios.create({
      baseURL: this.API_ROOT,
      timeout: 10000,
    });

    this.api.interceptors.request.use(
      config => {
        process.env.NODE_ENV !== 'production' && console.log(`API 요청 (액세스 토큰): ${this.accessToken}`);
        if (this.accessToken) config.headers['Authorization'] = `Bearer ${this.accessToken}`;
        return config;
      },
      e => e,
    );

    this.api.interceptors.response.use(
      res => res,
      async err => {
        const shouldRefreshToken = err.config && err.response && err.response.status === 401 && !err.config.__isRetry;
        const shouldShowServerError = err.response.status !== 500;

        if (shouldRefreshToken) {
          const errorCode = err.response?.data?.errorCode;

          if (errorCode === 'ACCESS_TOKEN_EXPIRED' || errorCode === 'NO_ACCESS_TOKEN') {
            return await this.silentRefreshTokenAndRetry(err);
          }

          if (errorCode === 'REFRESH_TOKEN_EXPIRED' || errorCode === 'NO_REFRESH_TOKEN') {
            return (window.location.href = this.loginPath);
          }
        }

        if (shouldShowServerError) {
          const errorMessage = err.response?.data?.message;
          errorMessage && sessionStorage.setItem('errorMessage', errorMessage);
        }

        throw err;
      },
    );
  }

  async get(path: string, params?: any) {
    return await this.api.get(path, { params });
  }

  async post(path: string, requestBody?: any) {
    return await this.api.post(path, requestBody);
  }

  async put(path: string, requestBody?: any) {
    return await this.api.put(path, requestBody);
  }

  async patch(path: string, requestBody?: any) {
    return await this.api.patch(path, requestBody);
  }

  async delete(path: string) {
    return await this.api.delete(path);
  }

  async refreshToken() {
    try {
      const { result } = (await this.post(this.tokenRefreshApiPath)).data;
      this.setAccessToken(result?.accessToken);
      return this.accessToken;
    } catch (e: any) {
      console.log('토큰 갱신에 실패하여 재로그인이 필요합니다.');
      window.location.href = this.loginPath;
      return;
    }
  }

  async silentRefreshTokenAndRetry(err) {
    try {
      const oldAccessToken = this.accessToken;
      err.config.__isRetry = true;
      const originalRequestConfig = err.config;
      delete originalRequestConfig.headers['Authorization'];

      const newAccessToken = await this.refreshToken();

      originalRequestConfig.headers['Authorization'] = `Baerer ${newAccessToken}`;
      process.env.NODE_ENV !== 'production' && console.log(`액세스 토큰이 갱신되었습니다.\nbefore: ${oldAccessToken}\n\nafter: ${newAccessToken}`);
      process.env.NODE_ENV !== 'production' && console.log(`토큰 갱신 후 이전 요청을 다시 시도하겠습니다. ${originalRequestConfig.url}`);

      return await this.api.request(originalRequestConfig);
    } catch (e: any) {
      process.env.NODE_ENV !== 'production' && console.log('silentRefreshTokenAndRetry 실패', e.message);
      throw e;
    }
  }

  async login(payload: any) {
    try {
      const { result } = (await this.post(this.loginApiPath, payload)).data;
      this.setAccessToken(result?.accessToken);
      return true;
    } catch (e: any) {
      console.error(e);
      return false;
    }
  }

  setAccessToken(token: string) {
    if (!token) {
      if (process.env.NODE_ENV !== 'production') console.log('새로운 액세스 토큰 등록에 실패했습니다. 토큰이 전달되지 않았습니다.');
      return;
    }
    this.accessToken = token;
    process.env.NODE_ENV !== 'production' && console.log(`새로운 액세스 토큰이 등록되었습니다. ${this.accessToken}`);
  }

  async getAccessToken() {
    if (!this.accessToken) await this.refreshToken();
    return this.accessToken;
  }

  setLoginPath(loginPath: string) {
    this.loginPath = loginPath;
  }
}

const API = new Api();
export default API;
